#==============================================================================
#
#  $Id: DateTime.py 697 2000-05-24 23:26:05Z mike $
#
"""
   Combination of date and time.  Same functionality as @spug.util.Date.Date
   but with hour, minute and second preserved.
"""
#
#  Copyright (C) 1999 Michael A. Muller
#
#  Permission is granted to use, modify and redistribute this code,
#  providing that the following conditions are met:
#
#  1) This copyright/licensing notice must remain intact.
#  2) If the code is modified and redistributed, the modifications must 
#  include documentation indicating that the code has been modified.
#  3) The author(s) of this code must be indemnified and held harmless
#  against any damage resulting from the use of this code.
#
#  This code comes with ABSOLUTELY NO WARRANTEE, not even the implied 
#  warrantee of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#
#  $Log: DateTime.py,v $
#  Revision 1.2  2000/05/24 23:26:05  mike
#  Fix for "forever future" constructor; fix for DST calculation problem.
#
#  Revision 1.1  2000/05/16 01:03:13  mike
#  Combination of Date and Time.
#
#
#==============================================================================

import time
from Date import Date

class DateTime(Date):
   """
      Class that stores a date and time.  Derives from @spug.util.Date.Date
      and supports all of its functionality.
   """
   
   def __init__(self, timeval = 0, other = None, year = 0, month = 0, day = 0,
                hour = 0, minute = 0, second = 0, foreverPast = 0,
                foreverFuture = 0
                ):
      """
         Note that all of *Date*s construction forms are supported except
         for *stdfmt*.
      """
      if year:
         Date.__init__(self, year = year, month = month, day = day)
         self.__hour = hour
         self.__minute = minute
         self.__second = second
      elif other:
         Date.__init__(self, other = other)
         self.__hour = other.__hour
         self.__minute = other.__minute
         self.__second = other.__second
      elif foreverPast:
         Date.__init__(self, foreverPast = 1)
         self.__hour = self.__minute = self.__second = 0
      elif foreverFuture:
         Date.__init__(self, foreverFuture = 1)
         self.__hour = self.__minute = self.__second = 0
      else:
         if not timeval:
            timeval = time.time()
         info = time.localtime(timeval)
         Date.__init__(self, year = info[0], month = info[1], day = info[2])
         self.__hour = info[3]
         self.__minute = info[4]
         self.__second = info[5]
      
   def __cmp__(self, other):
      """
         Overrides the date comparison operator to allow us to compare time.
      """
      return Date.__cmp__(self, other) or \
         cmp(self.__hour, other.__hour) or \
         cmp(self.__minute, other.__minute) or \
         cmp(self.__second, other.__second)

   def __str__(self):
      return self.formatNewStd() + \
         ' %02d:%02d:%02d' % (self.__hour, self.__minute, self.__second)
   
   def _toTime(self):
      """
         Overrides @spug.util.Date.Date._toTime() to include hour, minute
         and second.
      """
      y, m, d = self.ymd()
      return time.mktime(y, m, d, self.__hour, self.__minute, self.__second, 
                         0, 0, -1
                         )
      

   def getHour(self):
      "Returns the hour of the day (0-23)"
      return self.__hour
   
   def getMinute(self):
      "Returns the minute of the hour (0-59)"
      return self.__minute
   
   def getSecond(self):
      "Returns the second"
      return self.__second
   
