/*===========================================================================*\

   $Id$

   Copyright (C) 2007 Michael A. Muller

   Permission is granted to use, modify and redistribute this code,
   providing that the following conditions are met:
 
   1) This copyright/licensing notice must remain intact.
   2) If the code is modified and redistributed, the modifications must 
   include documentation indicating that the code has been modified.
   3) The author(s) of this code must be indemnified and held harmless
   against any damage resulting from the use of this code.

   This code comes with ABSOLUTELY NO WARRANTEE, not even the implied 
   warrantee of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

\*===========================================================================*/

#ifndef AUDIOPLAYER_H
#define AUDIOPLAYER_H

#include <spug/RCBase.h>
#include <spug/RCPtr.h>
#include "Moveable.h"

class Camera;
SPUG_RCPTR(AudioPlayer);

/**
 * Interface for audio players.
 */
class AudioPlayer : public virtual spug::RCBase {

   public:
      /**
       * Sets the moveable associated with the audio source.  This will define
       * the source's position in space.  If the moveable is null, the audio
       * source is not positional - it will have maximum volume evenly
       * distributed across all speakers.
       */
      virtual void setMoveable(const MoveablePtr &moveable) = 0;

      /** 
       * Start playing the audio source.
       * @throws InvalidStateException if the player is already playing.
       */
      virtual void start() = 0;

      /**
       * Stop playing the audio source.
       * @throws InvalidStateException if the player is not playing.
       */
      virtual void stop() = 0;

      /**
       * Called when the player finishes playing.
       */
      virtual void handleStopped() = 0;
};

#endif
