
#include "Model.h"
#include <spug/STLIteratorImpl.h>
#include "SphereObj.h"

using namespace spug;

Model::Model() {}

void Model::addPoly(const PolyObjPtr &poly) {
   polys.push_back(poly);
}

void Model::addSphere(const SphereObjPtr &sphere) {
   spheres.push_back(sphere);
}

void Model::render(Scene &scene) {
   // render each polygon in the model
   for (PolyVec::iterator iter = polys.begin(); iter != polys.end(); ++iter)
      (*iter)->render(scene);

   for (SphereVec::iterator iter = spheres.begin(); iter != spheres.end();
        ++iter
        )
      (*iter)->render(scene);
}


Model::PolyIter Model::getPolyIterator() const {
   typedef 
      STLIteratorImpl<const PolyObjPtr, 
		      const Model::PolyVec, 
		      Model::PolyVec::const_iterator> 
      PolyObjIterImpl;
   return Model::PolyIter(PolyObjIterImpl::alloc(polys));
}

AxisBox Model::getBoundingBox() const {
   // create an inside out box (so the first polygon will exceed all
   // boundaries)
   AxisBox box(false);

   for (PolyVec::const_iterator iter = polys.begin(); iter != polys.end(); 
        ++iter
        ) {
      AxisBox polyBox = (*iter)->getBoundingBox();
      box.expandTo(polyBox);
   }

   return box;
}
